use std::sync::Arc;

use async_trait::async_trait;
use axum::{
    extract::FromRequestParts,
    http::{request::Parts, StatusCode},
};
use figment::{
    providers::{Env, Format, Toml, Yaml},
    Figment,
};
use serde::{Deserialize, Serialize};

use crate::AppCtx;

#[derive(Debug, Clone, Deserialize, Serialize, Default)]
pub struct Config {
    pub hmac_key: String,
}

#[async_trait]
impl FromRequestParts<AppCtx> for Arc<Config> {
    type Rejection = StatusCode;

    async fn from_request_parts(
        _parts: &mut Parts,
        state: &AppCtx,
    ) -> Result<Self, Self::Rejection> {
        Ok(state.config.clone())
    }
}

pub fn make_config() -> Result<Config, anyhow::Error> {
    Ok(Figment::new()
        .join(Toml::file("Config.toml"))
        .merge(Yaml::file("Config.yml"))
        .merge(Yaml::file("Config.yaml"))
        .merge(Env::prefixed("OMEGA__"))
        .extract()?)
}
