use std::string::FromUtf8Error;

use thiserror::Error;

pub type Result<T, E = Error> = core::result::Result<T, E>;

#[derive(Error, Debug)]
pub enum Error {
    #[error("Error accessing database: {0}")]
    DatabaseError(#[from] sqlx::Error),
    #[error("Error migrating database: {0}")]
    MigrationError(#[from] sqlx::migrate::MigrateError),
    #[error("Error parsing UUID: {0}")]
    UuidError(#[from] sqlx::types::uuid::Error),
    #[error("String wasn't valid UTF8: {0}")]
    Utf8Error(#[from] FromUtf8Error),
}
